@@ -2,8 +2,10 @@
 import os
 import shutil
 import difflib
+import pprint
 import web_minify.processor
 from web_minify.files import generate_file_list
+from web_minify.handlers import HandlerManager
 
 import logging
 
@@ -40,11 +42,27 @@ def color_diff(diff):
             yield line
 
 
+def _test_handlers():
+    settings = {"mode": "test", "input": "", "output": "", "verbose": False}
+    hm = HandlerManager(settings)
+    assert hm is not None
+    extensions = hm.get_supported_extensions()
+    assert len(extensions) > 0
+    for ext in extensions:
+        handler = hm.get_handler_by_extension(ext)
+        assert handler is not None
+        handler.is_binary()
+        handler.extensions()
+        handler.process("")
+    inexistant_h = hm.get_handler_by_extension("inexistant")
+    assert inexistant_h is None
+
+
 def test_minify_regressions():
     return regressions_work("minify")
 
 
-def test_beautify_regressions():
+def _test_beautify_regressions():
     return regressions_work("beautify")
 
 
@@ -59,7 +77,22 @@ def regressions_work(mode):
     # 1. Clear old result if any
     # clear_output()
     # 2. Run tool on input
-    settings = {"mode": mode, "input": input_path, "output": output_path, "verbose": False}
+    # fmt:off
+    settings = {
+        "mode": mode
+        , "input": input_path
+        , "output": output_path
+        , "verbose": False
+        , "overwrite": True
+        , "force": True
+        #, "disable_type_css": True
+        #, "disable_type_html": True
+        #, "disable_type_sass": True
+        #, "disable_type_png": True
+        #, "disable_type_jpeg": True
+        #, "disable_type_svg": True
+    }
+    # fmt:on
     processor = web_minify.processor.Processor(settings)
     res, msg = processor.sanity_checks()
     if not res:
@@ -89,7 +122,7 @@ def regressions_work(mode):
         with open(expected_file, "rb") if expected_is_binary else open(expected_file, "r", encoding="utf-8") as expected_handle:
             expected_content = expected_handle.read()
         if output_content != expected_content:
-            logger.warning(f"--- Output did not matchexpected for {rel}:")
+            logger.warning(f"--- Output did not match expected for {rel}:")
             diff = difflib.ndiff(output_content.splitlines(1), expected_content.splitlines(1))
             diff = color_diff(diff)
             logger.error("\n".join(diff))
