@@ -1,5 +1,9 @@
 # https://engineeringblog.yelp.com/2017/06/making-photos-smaller.html
 
+import logging
+
+logger = logging.getLogger(__name__)
+
 
 def jpeg_minify(jpeg, settings: dict):
     """Minify JPEG main function."""
@@ -14,16 +18,19 @@ class Handler:
     def __init__(self, settings: dict):
         self.settings = settings
 
+    @classmethod
     def is_binary(self):
         return True
 
+    @classmethod
     def extensions(self):
         return ["jpeg", "jpg"]
 
+    @classmethod
     def name(self):
         return "jpeg"
 
-    def process(self, raw: str):
+    def process(self, raw: str, name: str = None):
         mode = self.settings.get("mode")
         if "beautify" == mode:
             # TODO: Implement
