@@ -3,6 +3,10 @@
 
 # import pngquant
 
+import logging
+
+logger = logging.getLogger(__name__)
+
 
 def png_minify(png, settings: dict):
     """Minify PNG main function."""
@@ -18,16 +22,19 @@ class Handler:
     def __init__(self, settings: dict):
         self.settings = settings
 
+    @classmethod
     def is_binary(self):
         return True
 
+    @classmethod
     def extensions(self):
         return ["png"]
 
+    @classmethod
     def name(self):
         return "png"
 
-    def process(self, raw: str):
+    def process(self, raw: str, name: str = None):
         mode = self.settings.get("mode")
         if "beautify" == mode:
             # TODO: Implement
