@@ -1,5 +1,9 @@
 from .scour import scourString
 
+import logging
+
+logger = logging.getLogger(__name__)
+
 
 def svg_minify(svg, settings: dict):
     """Minify SVG main function."""
@@ -13,16 +17,19 @@ class Handler:
     def __init__(self, settings: dict):
         self.settings = settings
 
+    @classmethod
     def is_binary(self):
         return False
 
+    @classmethod
     def extensions(self):
         return ["svg"]
 
+    @classmethod
     def name(self):
         return "svg"
 
-    def process(self, raw: str):
+    def process(self, raw: str, name: str = None):
         mode = self.settings.get("mode")
         if "beautify" == mode:
             # TODO: Implement
