@@ -77,7 +77,7 @@ lexicon = [("float", r"[-+]?(?:(?:[0-9]*\.[0-9]+)|(?:[0-9]+\.?))(?:[Ee][-+]?[0-9
 
 
 class Lexer(object):
-    """ Break SVG path data into tokens.
+    """Break SVG path data into tokens.
 
     The SVG spec requires that tokens are greedy. This lexer relies on Python's
     regexes defaulting to greediness.
@@ -96,7 +96,7 @@ class Lexer(object):
         self.regex = re.compile(self.regex_string)
 
     def lex(self, text):
-        """ Yield (token_type, str_data) tokens.
+        """Yield (token_type, str_data) tokens.
 
         The last token will be (EOF, None) where EOF is the singleton object
         defined in this module.
@@ -114,7 +114,7 @@ svg_lexer = Lexer(lexicon)
 
 
 class SVGTransformationParser(object):
-    """ Parse SVG transform="" data into a list of commands.
+    """Parse SVG transform="" data into a list of commands.
 
     Each distinct command will take the form of a tuple (type, data). The
     `type` is the character string that defines the type of transformation in the
@@ -138,8 +138,7 @@ class SVGTransformationParser(object):
         self.number_tokens = list(["int", "float"])
 
     def parse(self, text):
-        """ Parse a string of SVG transform="" data.
-        """
+        """Parse a string of SVG transform="" data."""
         gen = self.lexer.lex(text)
         next_val_fn = partial(next, *(gen,))
 
