@@ -1,38 +1,42 @@
 #!/usr/bin/env python3
 
 import argparse
-
-from enum import Enum
+from argparse import RawDescriptionHelpFormatter
 
 from .processor import Processor
 from .log import setup_logging
 
+from pathos.multiprocessing import cpu_count
+
 import os
+import sys
+
+from .modes import Mode
+from .handlers import HandlerManager
 
 from . import __version__
 
 logger = setup_logging(__name__)
 
+nproc = cpu_count()
+
 ##############################################################################
 
 
-class Mode(Enum):
-    minify = "minify"
-    beautify = "beautify"
-
-    def __str__(self):
-        return self.value
-
-
 def parse_arguments(defaults):
     """Build and return a command line agument parser."""
     parser = argparse.ArgumentParser(
         description=__doc__,
-        epilog="""web-minify:
-    Process all CSS/SASS/HTML/JS/SVG/PNG/JPEG found at input, either
-    in single file or recursively Applies transformation to each file according to selected mode:
-    
-    Mode = minify:
+        formatter_class=RawDescriptionHelpFormatter,
+        epilog=f"""
+    ---------------------------------------------------------
+
+    Process all CSS/SASS/HTML/JS/SVG/PNG/JPEG found at input,
+    either in single file or recursively.
+    Will apply transformation to each file according to
+    selected mode:
+
+    Mode = minify (for optimizing production):
      + Compile (SASS -> CSS)
      + Strip whitespace
      + Strip comments
@@ -42,22 +46,24 @@ def parse_arguments(defaults):
      + Add timestamp (CSS, HTML)
      + Obfuscate (CSS, JS)
      + Hash
-     
-    Mode = beautify:
+
+    Mode = beautify (for aiding development):
      + lint (JS, HTML)
      + normalize format (JS, HTML)
-     
+
     Provides some options for processing:
     + process files in-place
     + process files renaming to new form
     + compress results to .gz
-    
-    Available variables in format string are:
-    + {EXT}
-    + {HASH}
-    + {PATH}
-    + {BASE}
 
+    Available variables in format string are:
+    + {{EXT}}
+    + {{HASH}}
+    + {{PATH}}
+    + {{BASE}}
+ 
+    web-minify version {__version__}
+ 
 """,
     )
 
@@ -69,19 +75,18 @@ def parse_arguments(defaults):
     _option_group_general.add_argument("--on-change", action="store_true", help="Allow overwrite files only on source changed (detected by modify time).", default=defaults.get("on_change"))
     _option_group_general.add_argument("--verbose", action="store_true", help="Show output during processing.", default=defaults.get("verbose"))
     _option_group_general.add_argument("--force", action="store_true", help="Overwrite even if destination exists and is newer.", default=defaults.get("force"))
-    _option_group_general.add_argument("--zipy", action="store_true", help="GZIP Processed files as '*.gz'.", default=defaults.get("zipy"))
+    _option_group_general.add_argument("--nproc", action="store", type=int, dest="nproc", default=nproc, metavar="NUM", help=f"Set number of cores for multiprocessing (default is number of cores available which is {nproc} on this machine)")
+    _option_group_general.add_argument("--gzip", action="store_true", help="GZIP Processed files as '*.gz'.", default=defaults.get("gzip"))
 
-    _option_group_general.add_argument("--disable-js", action="store_true", dest="disable_js", default=False, help="Copy .js files verbatim instead of processing")
+    handlers = HandlerManager()
+    valid_extensions = handlers.get_supported_extensions()
+    handler_names = handlers.get_handler_names()
 
-    _option_group_general.add_argument("--disable-css", action="store_true", dest="disable_css", default=False, help="Copy .css files verbatim instead of processing")
+    for name in set(handler_names):
+        _option_group_general.add_argument(f"--disable-type-{name}", action="store_true", dest=f"disable_{name}", default=False, help=f"Copy {name} files verbatim instead of processing them for given type")
 
-    _option_group_general.add_argument("--disable-html", action="store_true", dest="disable_html", default=False, help="Copy .html files verbatim instead of processing")
-
-    _option_group_general.add_argument("--disable-svg", action="store_true", dest="disable_svg", default=False, help="Copy .svg files verbatim instead of processing")
-
-    _option_group_general.add_argument("--disable-png", action="store_true", dest="disable_png", default=False, help="Copy .png files verbatim instead of processing")
-
-    _option_group_general.add_argument("--disable-jpeg", action="store_true", dest="disable_jpeg", default=False, help="Copy .jpeg files verbatim instead of processing")
+    for name in set(valid_extensions):
+        _option_group_general.add_argument(f"--disable-suffix-{name}", action="store_true", dest=f"disable_{name}", default=False, help=f"Copy {name} files verbatim instead of processing them for given filename suffix")
 
     _option_group_general.add_argument("--output", type=str, help="Path to local output (file or folder).", default=defaults.get("output"))
     _option_group_general.add_argument("input", metavar="input", type=str, help="Path to local input (file or folder).", default=defaults.get("input"))
@@ -147,7 +152,8 @@ def main():
             , "on_change": True
             , "verbose": False
             , "force": False
-            , "zipy": False
+            , "nproc": nproc
+            , "gzip": False
             , "sort": False
             , "comments": False
             , "timestamp": False
@@ -165,12 +171,14 @@ def main():
         processor = Processor(settings)
         res, msg = processor.sanity_checks()
         if not res:
-            logger.error("Sanity check failed: {msg}, quiting...")
+            logger.error("Sanity check failed: {msg}, quitting...")
             sys.exit(1)
         if verbose:
             logger.info("Sanity check passed!")
-        if not processor.process_files():
-            logger.error("Processing failed, quiting...")
+        logger.info(f"main.NPROC: {nproc}")
+        status, msg = processor.process_files()
+        if not status:
+            logger.error(f"Processing failed with '{msg}', quitting...")
             sys.exit(1)
         if verbose:
             logger.info("Processing completed successfully!")
